/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
 * Namespace class for some static parsing-related methods.
 */

#ifndef nsParserUtils_h__
#define nsParserUtils_h__

#include "nsString.h"
class nsIAtom;

class nsParserUtils {
public:
  /**
   * This will parse aSource, to extract the value of the pseudo attribute
   * with the name specified in aName. See
   * http://www.w3.org/TR/xml-stylesheet/#NT-StyleSheetPI for the specification
   * which is used to parse aSource.
   *
   * @param aSource the string to parse
   * @param aName the name of the attribute to get the value for
   * @param aValue [out] the value for the attribute with name specified in
   *                     aAttribute. Empty if the attribute isn't present.
   * @return PR_TRUE  if the attribute exists and was successfully parsed.
   *         PR_FALSE if the attribute doesn't exist, or has a malformed
   *                  value, such as an unknown or unterminated entity.
   */
  static PRBool
  GetQuotedAttributeValue(const nsString& aSource, nsIAtom *aName,
                          nsAString& aValue);

  /**
   * This will parse aSource, to extract the name of the pseudo attribute
   * at the specified index. See
   * http://www.w3.org/TR/xml-stylesheet/#NT-StyleSheetPI for the specification
   * which is used to parse aSource.
   *
   * @param aSource the string to parse
   * @param aIndex the index of the attribute to get the value for
   * @param aName [out] the name for the attribute with specified index.
   *                    Empty if there aren't enough attributes.
   * @return PR_TRUE if parsing succeeded, even if there aren't enough
   *                 attributes.
   *         PR_FALSE if parsing failed.
   */
  static PRBool
  GetQuotedAttrNameAt(const nsString& aSource, PRUint32 aIndex,
                      nsAString& aName);

  static PRBool
  IsJavaScriptLanguage(const nsString& aName, PRUint32 *aVerFlags);

  static void
  SplitMimeType(const nsAString& aValue, nsString& aType,
                nsString& aParams);
};

#endif // nsParserUtils_h__



